import processing.serial.*;
import controlP5.*;

Serial port = null;

ControlP5 controlP5;
Textarea term;
Textfield input;

// port vars
String LIST_PORTS = "list ports";
String OPEN_PORT = "open port";
String CLOSE_PORT = "close port";
String LOAD_FILE = "load";
String SAVE_FILE = "save";
int BAUD = 115200;

String fileName = "pic_output.txt";

// gui vars
int width = 640;
int height = 480;
int TXTFLD = 40;
int SCROLL = 10;

void setup() {
  size(width, height);

  controlP5 = new ControlP5(this);

  // create the terminal textarea
  term = controlP5.addTextarea("term", "", 0, 0, width-SCROLL, height-TXTFLD);
  term.setColor(0);
  term.setColorBackground(255);
  term.showScrollbar();
  term.scroll(1);

  // create the textfield
  input = controlP5.addTextfield("input", 0, height-TXTFLD, width, TXTFLD);
  input.setFocus(true);
}

void draw() {
  input.setFocus(true);
}

public void input(String txt) {
  String tarea = term.text();
  tarea += ("<< " + txt + "\r\n");

  if (isList(txt)) {
    // list ports
    String[] ports = Serial.list();
    for (int i = 0; i < ports.length; i++) {
      tarea += ("\t [" + i + "] " + ports[i] + "\r\n");
    }
  }
  else if (isOpen(txt)) {
    // open port
    if (port != null) {
      port.stop();
      tarea += ("\t closed previous port.\r\n");
    }

    int i = int(txt.substring(OPEN_PORT.length()+1));
    String[] ports = Serial.list();
    if (i > 0 && i < ports.length) {
      port = new Serial(this, ports[i], BAUD);
      port.bufferUntil('\n');
      tarea += ("\t " + ports[i] + " opened.\r\n");
    }
  }
  else if (isClose(txt)) {
    // close port
    if (port != null) {
      port.stop();
      port = null;
      tarea += ("\t closed port.\r\n");
    }
  }
  else if (isLoad(txt)) {
    // load trajectory
    String file = txt.substring(LOAD_FILE.length()+1);
    String[] lns = loadStrings(file);
    if (lns != null) {
      // concatenate file into one long space delimited string
      file = "";
      for (int i = 0; i < lns.length; i++) {
        file += (trim(lns[i]) + " ");
      }

      // send trajectory to pic
      tarea += portWrite("load " + file + "\r\n");
    }
    else {
      tarea += ("\t invalid usage: save [data] [file name].\r\n");
    }
  }
  else if (isSave(txt)) {
    // save data
    String[] cmd = splitTokens(txt);
    if (cmd.length == 3) {
      // update file name
      fileName = cmd[2];
      tarea += portWrite(cmd[0] + " " + cmd[1] + "\r\n");
    }
    else {
      tarea += ("\t could not open file; is it in the Terminal/data folder?.\r\n");
    }
  }
  else {
    // send to pic
    tarea += portWrite(txt + "\r\n");
  }

  term.setText(tarea);
}

boolean isList(String txt) {
  return LIST_PORTS.equals(txt);
}

boolean isOpen(String txt) {
  int n = OPEN_PORT.length();
  if (n >= txt.length()+1) {
    // not enough characters
    return false;
  }

  String cmd = txt.substring(0, n);
  return OPEN_PORT.equals(cmd);
}

boolean isClose(String txt) {
  return CLOSE_PORT.equals(txt);
}

boolean isLoad(String txt) {
  int n = LOAD_FILE.length();
  if (n >= txt.length()+1) {
    // not enough characters
    return false;
  }

  String cmd = txt.substring(0, n);
  return LOAD_FILE.equals(cmd);
}

boolean isSave(String txt) {
  int n = SAVE_FILE.length();
  if (n >= txt.length()+1) {
    // not enough characters
    return false;
  }

  String cmd = txt.substring(0, n);
  return SAVE_FILE.equals(cmd);
}

void saveSignal(String txt) {
  String tarea = term.text();
  String[] lns = splitTokens(txt);
  int rows = int(lns[1]);
  int cols = (lns.length - 2) / rows;
  int tok;
  String[] file = new String[cols];

  tok = 2;
  for (int i = 0; i < cols; i++) {
    file[i] = "";
    for (int j = 0; j < rows; j++) {
      file[i] += lns[tok] +  " ";
      tok++;
    }
  }
  saveStrings(fileName, file);

  tarea += ("\t saved signal to file: " + fileName + ".\r\n");
  term.setText(tarea);
}

String portWrite(String txt) {
  if (port != null) {
    port.write(txt);
    return "";
  }

  return ("\t no port is open, cannot send cmd.\r\n");
}

void serialEvent(Serial myPort) {
  String tarea = term.text();
  String txt = myPort.readString();

  if (isSave(txt)) {
    // save trajectory
    saveSignal(txt);
  }
  else {
    tarea += (">> " + txt);
    term.setText(tarea);
  }
}
